#import <ARKit/ARKit.h>
#import "ARContext.h"

@interface ARContext()
@property (nonatomic, strong) ARWorldTrackingConfiguration *worldCofigution;
@end

@implementation ARContext

- (instancetype)init{
    self = [super init];
    if (self) {
        
        self.worldCofigution = [[ARWorldTrackingConfiguration alloc] init];
        self.worldCofigution.autoFocusEnabled = NO;
        self.worldCofigution.lightEstimationEnabled = NO;
        
        self.session = [[ARSession alloc] init];
        self.scnView = [[ARSCNView alloc] init];
        self.scnView.session = self.session;
        self.scnView.automaticallyUpdatesLighting = YES;
        self.scnView.autoenablesDefaultLighting = YES;
        self.scnView.debugOptions = ARSCNDebugOptionShowWorldOrigin | ARSCNDebugOptionShowFeaturePoints;
    }
    return self;
}

- (void)setSessionDelegate:(id<ARSessionDelegate>)sessionDelegate{
    self.session.delegate = sessionDelegate;
}
- (id<ARSessionDelegate>) getSessionDelegate{
    return self.session.delegate;
}
- (void)setViewDelegate:(id<ARSCNViewDelegate>)viewDelegate{
    self.scnView.delegate = viewDelegate;
}
- (id<ARSCNViewDelegate>) getViewDelegate{
    return self.scnView.delegate;
}

- (void)addChildNode:(SCNNode *)child{
    simd_float4x4 transform = matrix_identity_float4x4;
    transform.columns[3].x = child.position.x;
    transform.columns[3].y = child.position.y;
    transform.columns[3].z = child.position.z;
    ARAnchor *anchor = [[ARAnchor alloc] initWithTransform:transform];
    [self.scnView.session addAnchor:anchor];
    [self.scnView.scene.rootNode addChildNode:child];
}

- (void)removeChildNodeWithName:(NSString *)name{
    for (SCNNode *subNode in self.scnView.scene.rootNode.childNodes) {
        if ([subNode.name isEqualToString:name]) {
            [subNode removeFromParentNode];
        }
    }
}

- (void)startRuning{
    DLog(@"start Runing location_time%@",[NSDate date]);
    [self.scnView.session runWithConfiguration:self.worldCofigution];
}

- (void)pause{
    [self.scnView.session pause];
}

-(void)session:(ARSession *)session didFailWithError:(NSError *)error{
    DLog(@"%@",error.userInfo);
}

- (void)dealloc
{
    NSLog(@"arContext release");
}
@end
