#import "ARViewController.h"
#import "ARContext.h"
#import <ligsdk/Util.h>

@interface ARViewController ()<ARSessionDelegate,ARSCNViewDelegate>
{
    BOOL isCalibrated;
}
@property (nonatomic) ARContext *context;
@property (nonatomic,assign) simd_float4x4 worldOrigin;
@property (nonatomic,assign) bool isWorldOriginSetted;
@end

@implementation ARViewController

-(instancetype)initWithWorldOrigin:(simd_float4x4)transform {
    self = [super init];
    if(self){
        self.worldOrigin = transform;
        self.isWorldOriginSetted = true;
    }
    return self;
}
-(instancetype)init{
    self = [super init];
    if(self){
        self.isWorldOriginSetted = false;
        [self.view addSubview:[UIButton buttonWithType:UIButtonTypeSystem]];
    }
    return self;
}

#pragma mark - life cycle
-(void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self.context pause];
    self.context = nil;
    [[UIApplication sharedApplication] setIdleTimerDisabled:NO];
    DLog(@"this class dealloc");
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [[UIApplication sharedApplication] setIdleTimerDisabled:YES];
    self.view.backgroundColor = [UIColor clearColor];
    [self initARContext];
}
- (void)initARContext{
    self.context = [[ARContext alloc] init];
    self.context.scnView.frame = CGRectMake(0,0,SCREEN_WIDTH,SCREEN_HEIGHT);
    [self.view addSubview:self.context.scnView];
    self.context.viewDelegate = self;
    self.context.sessionDelegate = self;
    [self.context startRuning];
    if(self.isWorldOriginSetted){
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(2 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [self.context.session setWorldOrigin:self.worldOrigin];
            SCNNode *node = [SCNNode nodeWithGeometry:[SCNSphere sphereWithRadius:10]];
            node.geometry.firstMaterial.diffuse.contents = [UIImage imageNamed:@"demo_ball"];
            node.worldPosition = SCNVector3Make(0,0,2);
            [self.context addChildNode:node];
        });
    }
}

#pragma mark - ARSessionDelegate
- (void)session:(ARSession *)session didAddAnchors:(NSArray<ARAnchor *> *)anchors {
    if (@available(iOS 12.0, *)) {
        for (int i = 0; i < [anchors count]; i++) {
            
        }
    }
}

- (void)session:(ARSession *)session didUpdateAnchors:(nonnull NSArray<ARAnchor *> *)anchors {
    if (@available(iOS 12.0, *)) {
        for (int i = 0; i < [anchors count]; i++) {
            
        }
    }
}

-(void)session:(ARSession *)session didUpdateFrame:(ARFrame *)frame {
    if (!isCalibrated)
    {
        [LightUtil getARIntrinsics:frame.camera];
        NSLog(@"[LiGSDK] AR Session is Calibrated.");
        isCalibrated = true;
    }
}

#pragma mark - arkit error
- (void)sessionWasInterrupted:(ARSession *)session {
    DLog(@"sessionWasInterrupted");
}

- (void)sessionInterruptionEnded:(ARSession *)session {
    DLog(@"sessionInterruptionEnded");
}

- (BOOL)sessionShouldAttemptRelocalization:(ARSession *)session {
    DLog(@"sessionShouldAttemptRelocalization");
    return NO;
}

- (void)session:(ARSession *)session didFailWithError:(NSError *)error {
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self.context.scnView removeFromSuperview];
        [self initARContext];
    });
}

@end
