#import "Preview.h"
#include "simd/simd.h"

#define ICON_WH 60
#define lockDistance 20
#define MAX_ICONS 8
@interface Preview ()
@property(nonatomic, strong) NSMutableArray *icons;
@property(nonatomic, assign) CGPoint c;
@end

@implementation Preview

- (instancetype)initWithFrame:(CGRect)frame {
    if (self = [super initWithFrame:frame]) {
        self.c = CGPointMake(frame.size.width/2, frame.size.height/2);
        self.icons = [NSMutableArray array];
        for (int i = 0; i < MAX_ICONS; i++) {
            UIImageView *imageView = [[UIImageView alloc] init];
            imageView.tag = i;
            imageView.frame = CGRectMake(0,0,ICON_WH, ICON_WH);
            [self.icons addObject:imageView];
        }        
    }
    return self;
}

- (void)showIconsWithItems:(NSArray <Light *> *)items
      focuLightLabelHandle:(FocusedLightLabelBlock)focuLightLabelHandle{
    [self clearIcons];

    dispatch_async(dispatch_get_main_queue(), ^{
        for (int i = 0; i < MIN(items.count,MAX_ICONS); i++) {
            UIImageView *scanView = self.icons[i];
            Light *item = (Light *) items[i];

            // You can check light.status for more info
            // if (item.status == LightStatusReady) ...

            scanView.center = CGPointMake(item.screenX * SCREEN_WIDTH, item.screenY * SCREEN_HEIGHT/2);
            [self addSubview:scanView];
            if (item.ligId == 0) {
                [scanView setImage:[UIImage imageNamed:@"unknown"]];
                continue;
            } else {
                [scanView setImage:[UIImage imageNamed:@"identified"]];
            }

            if (lockDistance > simd_length(simd_make_float2(scanView.center.x - self.c.x, scanView.center.y - self.c.y))) {
                scanView.center = self.c;
                focuLightLabelHandle(item);
            }
        }
    });
}

- (void)clearIcons {
    dispatch_async(dispatch_get_main_queue(), ^{
        for (UIView *subView in self.subviews) {
            if ([subView isKindOfClass:UIImageView.class]) {
                [subView removeFromSuperview];
                ((UIImageView *)subView).image = IMAGE(@"unknown");
            }
        }
    });
}

@end
