#import "ViewController.h"

#import <ligsdk/ligsdk.h>
#import "Preview.h"
#import <AVFoundation/AVFoundation.h>
#import "ARViewController.h"

@interface ViewController () <LightScannerDelegate>
{
    Preview *preview;
    LightScanner *scanner;
    BOOL started;
    __weak IBOutlet UILabel *txtSDK;
}
@end

@implementation ViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    scanner = [LightScanner sharedInstance];
    [scanner setDelegate:self];

    self.navigationController.navigationBarHidden = YES;
    
    UIImageView *baseImageview = [[UIImageView alloc] init];
    baseImageview.frame = self.view.bounds;
    [self.view addSubview:baseImageview];
    baseImageview.image = [UIImage imageNamed:@"bg_img"];
    self.view.backgroundColor = UIColor.blackColor;

    preview = [[Preview alloc] initWithFrame:CGRectMake(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT/2)];
    [self.view addSubview:preview];

    UIImageView *crossHair = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 255, 255)];
    crossHair.contentMode = UIViewContentModeScaleAspectFit;
    crossHair.image = IMAGE(@"scanner");
    crossHair.center = preview.center;
    [self.view addSubview:crossHair];

    if ([AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeVideo] == AVAuthorizationStatusNotDetermined)
    {
        [AVCaptureDevice requestAccessForMediaType:AVMediaTypeVideo completionHandler:^(BOOL granted) {
            if (!self->started)
            {
                [self startScanning];
            }
        }];
    }
    
    [SensorManager sharedInstance];

    NSString *version = [LightScanner getVersion];
    NSString *sdkText = [NSString stringWithFormat:@"SDK Version: %@", version];
    [txtSDK setText:sdkText];
    NSLog(@"%@", sdkText);
}

-(void)startScanning
{
    if (!started)
    {
        [scanner startWith:^(BOOL succeeded) {
            if (succeeded)
            {
                self->started = true;
            } else {
                NSLog(@"Start failed.");
            }
        }];
    }
}

-(void)stopScanning
{
    if (!started)
    {
        return;
    }

    [scanner stop];
    started = false;
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t) (0.05 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self->preview clearIcons];
    });
}

-(void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    
    if ([scanner isActive])
    {
        [self startScanning];
    }
}

-(void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
    
    [self stopScanning];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
}

-(void)ligDetectionOutputResult:(NSArray<Light *> *)resultArray error:(NSError *)error
{
    if (error) {
        // You can check error.code with ScanStatus enum constants
        // if (error.code == ScanStatusBadNetwork) ...
        
        NSLog(@"%@",error.description);
        return;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        if (resultArray.count <= 0) {
            [self->preview clearIcons];
        }else{
            [self->preview showIconsWithItems:resultArray focuLightLabelHandle:^(Light * _Nullable light) {
                if (light.z == 0) {
                    return;
                }

                [self stopScanning];
                simd_float4x4 transform = [[SensorManager sharedInstance] transformArray:light withGravity:TRUE];
                ARViewController *arview = [[ARViewController alloc] initWithWorldOrigin:transform];
                [self.navigationController pushViewController:arview animated:YES];
            }];
        }
    });
}

@end
